---
title: llarp/layers/flow/flow_identity.hpp

---

# llarp/layers/flow/flow_identity.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::layers](Namespaces/namespacellarp_1_1layers.md)**  |
| **[llarp::layers::flow](Namespaces/namespacellarp_1_1layers_1_1flow.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| struct | **[llarp::layers::flow::FlowIdentityPrivateKeys](Classes/structllarp_1_1layers_1_1flow_1_1FlowIdentityPrivateKeys.md)** <br>private keys used at the flow layer  |
| class | **[llarp::layers::flow::FlowIdentity](Classes/classllarp_1_1layers_1_1flow_1_1FlowIdentity.md)** <br>the local end of a one to one flow to a remote.  |




## Source code

```cpp
#pragma once

#include <oxenc/bt_producer.h>
#include <oxenc/bt_serialize.h>
#include <llarp/crypto/types.hpp>
#include <llarp/dht/key.hpp>
#include <string_view>

#include "flow_addr.hpp"
#include "flow_auth.hpp"
#include "flow_info.hpp"
#include "flow_state.hpp"
#include "flow_tag.hpp"
#include "flow_traffic.hpp"
#include "llarp/util/types.hpp"

namespace llarp::layers::flow
{

  struct FlowIdentityPrivateKeys
  {
    PQKeyPair sntrup;
    SecretKey identity;
    SecretKey encryption;
    PrivateKey derivedKey;

    static FlowIdentityPrivateKeys
    keygen();

    const FlowAddr&
    public_addr() const;
    const dht::Key_t&
    keyspace_location() const;

    void
    serialize(oxenc::bt_dict_producer&) const;
    void
    deserialize(oxenc::bt_dict_consumer&);

   private:
    mutable FlowAddr _root_pubkey;
    mutable dht::Key_t _derived_pubkey;
  };

  class FlowIdentity
  {
    const FlowIdentityPrivateKeys& _local_privkeys;

    FlowState _state;

   public:
    const FlowInfo flow_info;

    FlowIdentity(const FlowIdentity&) = delete;
    FlowIdentity(FlowIdentity&&) = delete;

    FlowIdentity(FlowAddr remote_addr, FlowTag flow_tag, const FlowIdentityPrivateKeys& local_keys);

    void
    async_ensure_flow(FlowHandshaker handshaker);

    void
    send_to_remote(std::vector<byte_t> dataum, FlowDataKind kind);
  };
}  // namespace llarp::layers::flow
```


-------------------------------

Updated on 2023-02-08 at 22:18:31 +0000
