---
title: llarp/layers/platform/platform_driver.hpp

---

# llarp/layers/platform/platform_driver.hpp



## Namespaces

| Name           |
| -------------- |
| **[llarp](Namespaces/namespacellarp.md)** <br>[crypto.hpp] |
| **[llarp::dns](Namespaces/namespacellarp_1_1dns.md)**  |
| **[llarp::vpn](Namespaces/namespacellarp_1_1vpn.md)**  |
| **[llarp::layers](Namespaces/namespacellarp_1_1layers.md)**  |
| **[llarp::layers::flow](Namespaces/namespacellarp_1_1layers_1_1flow.md)**  |
| **[llarp::layers::platform](Namespaces/namespacellarp_1_1layers_1_1platform.md)**  |

## Classes

|                | Name           |
| -------------- | -------------- |
| class | **[llarp::layers::platform::OSTraffic_IO_Base](Classes/classllarp_1_1layers_1_1platform_1_1OSTraffic__IO__Base.md)**  |
| class | **[llarp::layers::platform::PlatformDriver](Classes/classllarp_1_1layers_1_1platform_1_1PlatformDriver.md)** <br>base type that is responsible for bridging the os and the flow layer.  |




## Source code

```cpp
#pragma once

#include "addr_mapper.hpp"
#include "llarp/layers/platform/name_cache.hpp"
#include "name_resolver.hpp"
#include "platform_stats.hpp"
#include "traffic.hpp"
#include <llarp/config/config.hpp>
#include <llarp/net/ip_packet.hpp>
#include <llarp/net/ip_range.hpp>

#include <llarp/util/formattable.hpp>
#include <llarp/util/types.hpp>

#include <memory>
#include <optional>
#include <string>
#include <string_view>
#include <vector>

namespace llarp
{
  class RoutePoker;
}

namespace llarp::dns
{
  class Server;
}

namespace llarp::vpn
{
  class NetworkInterface;
}

namespace llarp::layers::flow
{
  class FlowLayer;
  class FlowAddr;
  struct FlowInfo;
}  // namespace llarp::layers::flow

namespace llarp::layers::platform
{
  class NameResolver;
  class PlatformDriver;

  // handles reading and writing os traffic, like ip packets or whatever embedded lokinet wants to
  // do.
  class OSTraffic_IO_Base
  {
    PlatformAddr _our_addr;
    std::vector<OSTraffic> _recv_queue;

    void got_ip_packet(net::IPPacket);

   protected:
    EventLoop_ptr _loop;
    std::shared_ptr<vpn::NetworkInterface> _netif;

   public:
    explicit OSTraffic_IO_Base(const EventLoop_ptr&);

    constexpr const PlatformAddr&
    our_platform_addr() const
    {
      return _our_addr;
    }

    void
    attach(
        std::shared_ptr<vpn::NetworkInterface> netif, std::shared_ptr<EventLoopWakeup> wakeup_recv);

    virtual std::vector<OSTraffic>
    read_platform_traffic();

    virtual void
    write_platform_traffic(std::vector<OSTraffic>&& traff);
  };

  class PlatformDriver_pimpl;

  struct platform_io_wakeup_handler;

  class PlatformDriver
  {
   protected:
    friend struct platform_io_wakeup_handler;

    const std::shared_ptr<PlatformDriver_pimpl> _impl;
    flow::FlowLayer& _flow_layer;

    std::shared_ptr<EventLoopWakeup> _os_send;
    std::shared_ptr<EventLoopWakeup> _os_recv;

    NetworkConfig _netconf;

    AddrMapper _addr_mapper;
    NameCache _name_cache;
    NameResolver _name_resolver;

    std::shared_ptr<llarp::dns::Server> _dns_server;

    OSTraffic_IO_Base _io;
    void
    on_io();

   public:
    PlatformDriver(const PlatformDriver&) = delete;
    PlatformDriver(PlatformDriver&&) = delete;

    PlatformDriver(
        PlatformDriver_pimpl* impl,
        flow::FlowLayer& flow_layer,
        NetworkConfig netconf,
        DnsConfig dnsconf);

    virtual void
    got_os_traffic(std::vector<OSTraffic> pkts);

    void
    map_remote(
        std::string name,
        std::string auth,
        std::vector<IPRange> ranges,
        bool is_exit,
        std::function<void(std::optional<flow::FlowInfo>, std::string)> result_handler);

    void
    map_exit(
        std::string name,
        std::string auth,
        std::vector<IPRange> ranges,
        std::function<void(bool, std::string)> result_handler);

    PlatformStats
    current_stats() const;
  };
}  // namespace llarp::layers::platform

namespace llarp
{
  template <>
  inline constexpr bool IsToStringFormattable<layers::platform::PlatformAddr> = true;
}
```


-------------------------------

Updated on 2023-02-08 at 22:18:31 +0000
